// disable warning message about converting double constant to float constant
#pragma warning(disable:4305)

#include <modelerdll.h>

#define BASE_ROTATION   1
#define LOWER_TILT      2
#define UPPER_TILT      3
#define UPPER_TWIST     4

MODEL("Robotarm");

bool init(void* args)
{
    scale( BASE_ROTATION, "base rotation", 0.0, 360.0, 0.1, 0.0 );
    scale( LOWER_TILT, "lower arm tilt", -60.0, 60.0, 0.1, 0.0 );
    scale( UPPER_TILT, "upper arm tilt", -120.0, 120.0, 0.1, 0.0 );
    scale( UPPER_TWIST, "upper arm twist", -45.0, 45.0, 0.1, 0.0 );
    
    return true;
}
    

void redraw(int drawmode)
{
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    /* define the viewing transform:  orient the model so that +z is
       up and +x is to the right. */
    glMatrixMode( GL_MODELVIEW );
    glLoadIdentity();
    glTranslated( 0.0, -4.0, -20.0 );
    glRotated( -90.0, 1.0, 0.0, 0.0 );

    /* turn the whole assembly around the z-axis. */
    glRotated( get_control_d( BASE_ROTATION ), 0.0, 0.0, 1.0 );

    /* place a box with the center of its bottom face at the origin. */
    glPushMatrix();
    glTranslated( -1.5, -1.0, 0.0 );
    box( 3.0, 2.0, 1.0 );
    glPopMatrix();

    /* move the origin to the top of the box, slightly offset from the
       center. */
    glTranslated( 0.5, 0.0, 1.0 );

    /* turn the coordinate system on the y-axis to tilt the lower arm
       around its base. */
    glPushMatrix();
    glRotated( get_control_d( LOWER_TILT ), 0.0, 1.0, 0.0 );

    /* place the cylinder that the lower arm is mounted on. */
    glPushMatrix();
      /* want a cylinder oriented along y, so turn coordinates 90
         degrees, making z into y. */
      glRotated( 90.0, 1.0, 0.0, 0.0 );
      /* translate so that the cylinder is centered at the origin. */
      glTranslated( 0.0, 0.0, -0.5 );
    
      diffuse_color( 0.6, 0.3, 0.3 );
      cylinder( 1.0, 0.5, 0.5 );
    glPopMatrix();

    /* place the lower arm, a box with its bottom face centered at the
       origin. */
    glPushMatrix();
      glTranslated( -0.25, -0.25, 0.0 );
      diffuse_color( 0.6, 0.6, 0.6 );
      box( 0.5, 0.5, 3.0 );
    glPopMatrix();

    /* move the coordinate system to the center of the lower arm's
       upper face. */
    glPushMatrix();
      glTranslated( 0.0, 0.0, 3.0 );
      /* twist the upper arm around the axis of the lower arm. */
      glRotated( get_control_d( UPPER_TWIST ), 0.0, 0.0, 1.0 );

      /* place a sphere at the joint. */
      diffuse_color( 0.6, 0.3, 0.3 );
      sphere( 0.5 );

      /* place another upper arm, twisted an equal amount in the
	 opposite direction. */
      glPushMatrix();
        glRotated( -get_control_d( UPPER_TILT ), 0.0, 1.0, 0.0 );
        diffuse_color( 0.7, 0.7, 0.7 );
        cylinder( 3.0, 0.25, 0.0 );
      glPopMatrix();
    
      /* place an upper arm, twisted down around the y axis. */ 
      glPushMatrix();
        glRotated( get_control_d( UPPER_TILT ), 0.0, 1.0, 0.0 );
        cylinder( 3.0, 0.25, 0.0 );

	glPushMatrix();
	  glTranslated( 0.0, 0.0, 3.0 );
	  ambient_color( 0.0, 0.0, 0.4 );
	  diffuse_color( 0.0, 0.4, 0.7 );
	  glRotated( get_time() * 90.0, 0.0, 0.0, 1.0 );
	  triangle( 0.0, 0.0, 0.0,
		    0.0, 0.0, 1.0,
		    2.0, 0.0, 0.5 );
	  triangle( 0.0, 0.0, 0.0,
		    0.0, 0.0, 1.0,
		    0.0, 2.0, 0.5 );
	  triangle( 0.0, 0.0, 0.0,
		    0.0, 2.0, 0.5,
		    2.0, 0.0, 0.5 );
	  triangle( 0.0, 0.0, 1.0,
		    2.0, 0.0, 0.5,
		    0.0, 2.0, 0.5 );
	  glPopMatrix();
	
      glPopMatrix();

    glPopMatrix();
    
    glPopMatrix();
}

